/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* Copyright IBM Corp. 2013
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 
#include "ioqoptimmaskdb2func.h"

void OptimMaskStr(   SQLUDF_CLOB          *pArg1,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_CLOB          *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR           pSqlstate[SQLUDF_SQLSTATE_LEN+1],
                     SQLUDF_CHAR           pFname[SQLUDF_FQNAME_LEN+1],
                     SQLUDF_CHAR           pFspecname[SQLUDF_SPECNAME_LEN+1],
                     SQLUDF_CHAR           pMsgtext[SQLUDF_MSGTEXT_LEN+1],
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_CHAR *pRetStr = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data , 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2String(pRetStr, 0, 0, pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         pRet->length = 0;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, &Retlen, NULL);
   
   pRet->length = (Retlen > (int)pRet->length)?pRet->length : Retlen;

   memcpy(pRet->data, pRetStr, pRet->length);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}

void OptimMaskStrUtf16(   
                     SQLUDF_DBCLOB        *pArg1,
                     SQLUDF_DBCLOB        *pParamString, 
                     SQLUDF_DBCLOB        *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR           pSqlstate[SQLUDF_SQLSTATE_LEN+1],
                     SQLUDF_CHAR           pFname[SQLUDF_FQNAME_LEN+1],
                     SQLUDF_CHAR           pFspecname[SQLUDF_SPECNAME_LEN+1],
                     SQLUDF_CHAR           pMsgtext[SQLUDF_MSGTEXT_LEN+1],
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_GRAPH *pRetStr = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data, 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2String(pRetStr, 0, 0, pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         pRet->length = 0;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, &Retlen, NULL);
   
   pRet->length = (Retlen > (int)pRet->length)?pRet->length : Retlen;

   memcpy(pRet->data, pRetStr, pRet->length*sizeof(ODPP_USHORT));
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}

void OptimMaskDate(  SQLUDF_VARCHAR       *pArg1,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_VARCHAR       *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_CHAR *pRetDateTime = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2DateTime(pArg1, 
                                                   *pArg1Ind, 
                                                    strlen(pArg1), 
                                                    UDF_DATE,
                                                    pDbInfo);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2DateTime(pRetDateTime, 0, 0,UDF_DATE,pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the aged date
   pDataOutArr->GetDateTime(&pRetDateTime, pRetInd, &Retlen);
   
   //copy the aged date to return buffer
   memcpy(pRet, pRetDateTime, Retlen);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}

void OptimMaskTS(    SQLUDF_VARCHAR       *pArg1,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_VARCHAR       *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_CHAR *pRetDateTime = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2DateTime(pArg1, 
                                                   *pArg1Ind, 
                                                    strlen(pArg1), 
                                                    UDF_TIMESTAMP,
                                                    pDbInfo);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2DateTime(pRetDateTime, 0, 0,UDF_TIMESTAMP,pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetDateTime(&pRetDateTime, pRetInd, &Retlen);
   
   memcpy(pRet, pRetDateTime, Retlen);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}

void OptimMaskInt64( SQLUDF_BIGINT        *pArg1,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_BIGINT        *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_BIGINT *pRetBigInt = NULL;

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2Number(pArg1, 
                                                  *pArg1Ind);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2Number(pRetBigInt, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the aged date
   pDataOutArr->GetInt64(&pRetBigInt, pRetInd);
   
   //copy the aged date to return buffer
   memcpy(pRet, pRetBigInt, sizeof(SQLUDF_BIGINT));
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}

void OptimMaskInt32( SQLUDF_INTEGER       *pArg1,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_INTEGER       *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_INTEGER *pRetInt = NULL;

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a Number class because the input is an integer
   *(pDataInArr + 0) = (CDB2Data*) new CDB2Number(pArg1, 
                                                  *pArg1Ind);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a Number because we are 
   //returning an integer
   pDataOutArr = (CDB2Data*) new CDB2Number(pRetInt, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the masked data
   pDataOutArr->GetInteger(&pRetInt, pRetInd);
   
   //copy the masked data to return buffer
   memcpy(pRet, pRetInt, sizeof(SQLUDF_INTEGER));
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}

void OptimMaskInt16( SQLUDF_SMALLINT      *pArg1,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_SMALLINT      *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_SMALLINT *pRetInt = NULL;

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a Number class because the input is a short integer
   *(pDataInArr + 0) = (CDB2Data*) new CDB2Number(pArg1, 
                                                  *pArg1Ind);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a Number because we are 
   //returning a short integer
   pDataOutArr = (CDB2Data*) new CDB2Number(pRetInt, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the masked data
   pDataOutArr->GetShort(&pRetInt, pRetInd);
   
   //copy the masked data to return buffer
   memcpy(pRet, pRetInt, sizeof(SQLUDF_SMALLINT));
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}

void OptimMaskDouble(SQLUDF_DOUBLE        *pArg1,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_DOUBLE        *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_DOUBLE *pRetDbl = NULL;

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2Number(pArg1, 
                                                  *pArg1Ind);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2Number(pRetDbl, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the aged date
   pDataOutArr->GetDouble(&pRetDbl, pRetInd);
   
   //copy the aged date to return buffer
   memcpy(pRet, pRetDbl, sizeof(SQLUDF_DOUBLE));
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}

void OptimMaskStr2(  SQLUDF_CLOB          *pArg1,
                     SQLUDF_CLOB          *pArg2,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_CLOB          *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pArg2Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_CHAR *pRetStr = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 3
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[3];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data , 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pArg2->data , 
                                                  *pArg2Ind, 
                                                  pArg2->length, 
                                                  pDbInfo);

   //This is a string class because the parameter string is always a char
   *(pDataInArr + 2) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2String(pRetStr, 0, 0, pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         3,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         pRet->length = 0;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, &Retlen, NULL);
   
   pRet->length = (Retlen > (int)pRet->length)?pRet->length : Retlen;

   memcpy(pRet->data, pRetStr, pRet->length);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete pDataOutArr;
}

void OptimMaskStr3(  SQLUDF_CLOB          *pArg1,
                     SQLUDF_CLOB          *pArg2,
                     SQLUDF_CLOB          *pArg3,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_CLOB          *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pArg2Ind,
                     SQLUDF_NULLIND       *pArg3Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_CHAR *pRetStr = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 4
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Character Argument (pArg3),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[4];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data , 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pArg2->data , 
                                                  *pArg2Ind, 
                                                  pArg2->length, 
                                                  pDbInfo);

   *(pDataInArr + 2) = (CDB2Data*) new CDB2String(pArg3->data , 
                                                  *pArg3Ind, 
                                                  pArg3->length, 
                                                  pDbInfo);

   //This is a string class because the parameter string is always a char
   *(pDataInArr + 3) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2String(pRetStr, 0, 0, pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         4,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         pRet->length = 0;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete *(pDataInArr + 3);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, &Retlen, NULL);
   
   pRet->length = (Retlen > (int)pRet->length)?pRet->length : Retlen;

   memcpy(pRet->data, pRetStr, pRet->length);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete *(pDataInArr + 3);
   delete pDataOutArr;
}

void OptimMaskStr4(  SQLUDF_CLOB          *pArg1,
                     SQLUDF_CLOB          *pArg2,
                     SQLUDF_CLOB          *pArg3,
                     SQLUDF_CLOB          *pArg4,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_CLOB          *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pArg2Ind,
                     SQLUDF_NULLIND       *pArg3Ind,
                     SQLUDF_NULLIND       *pArg4Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_CHAR *pRetStr = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 5
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Character Argument (pArg3),
   //Character Argument (pArg4),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[5];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data , 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pArg2->data , 
                                                  *pArg2Ind, 
                                                  pArg2->length, 
                                                  pDbInfo);

   *(pDataInArr + 2) = (CDB2Data*) new CDB2String(pArg3->data , 
                                                  *pArg3Ind, 
                                                  pArg3->length, 
                                                  pDbInfo);

   *(pDataInArr + 3) = (CDB2Data*) new CDB2String(pArg4->data , 
                                                  *pArg4Ind, 
                                                  pArg4->length, 
                                                  pDbInfo);

   //This is a string class because the parameter string is always a char
   *(pDataInArr + 4) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2String(pRetStr, 0, 0, pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         5,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         pRet->length = 0;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete *(pDataInArr + 3);
      delete *(pDataInArr + 4);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, &Retlen, NULL);
   
   pRet->length = (Retlen > (int)pRet->length)?pRet->length : Retlen;

   memcpy(pRet->data, pRetStr, pRet->length);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete *(pDataInArr + 3);
   delete *(pDataInArr + 4);
   delete pDataOutArr;
}

void OptimMaskStr5(  SQLUDF_CLOB          *pArg1,
                     SQLUDF_CLOB          *pArg2,
                     SQLUDF_CLOB          *pArg3,
                     SQLUDF_CLOB          *pArg4,
                     SQLUDF_CLOB          *pArg5,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_CLOB          *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pArg2Ind,
                     SQLUDF_NULLIND       *pArg3Ind,
                     SQLUDF_NULLIND       *pArg4Ind,
                     SQLUDF_NULLIND       *pArg5Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_CHAR *pRetStr = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 6
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Character Argument (pArg3),
   //Character Argument (pArg4),
   //Character Argument (pArg5),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[6];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data , 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pArg2->data , 
                                                  *pArg2Ind, 
                                                  pArg2->length, 
                                                  pDbInfo);

   *(pDataInArr + 2) = (CDB2Data*) new CDB2String(pArg3->data , 
                                                  *pArg3Ind, 
                                                  pArg3->length, 
                                                  pDbInfo);

   *(pDataInArr + 3) = (CDB2Data*) new CDB2String(pArg4->data , 
                                                  *pArg4Ind, 
                                                  pArg4->length, 
                                                  pDbInfo);

   *(pDataInArr + 4) = (CDB2Data*) new CDB2String(pArg5->data , 
                                                  *pArg5Ind, 
                                                  pArg5->length, 
                                                  pDbInfo);

   //This is a string class because the parameter string is always a char
   *(pDataInArr + 5) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2String(pRetStr, 0, 0, pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         6,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         pRet->length = 0;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete *(pDataInArr + 3);
      delete *(pDataInArr + 4);
      delete *(pDataInArr + 5);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, &Retlen, NULL);
   
   pRet->length = (Retlen > (int)pRet->length)?pRet->length : Retlen;

   memcpy(pRet->data, pRetStr, pRet->length);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete *(pDataInArr + 3);
   delete *(pDataInArr + 4);
   delete *(pDataInArr + 5);
   delete pDataOutArr;
}

void OptimMaskStr6(  SQLUDF_CLOB          *pArg1,
                     SQLUDF_CLOB          *pArg2,
                     SQLUDF_CLOB          *pArg3,
                     SQLUDF_CLOB          *pArg4,
                     SQLUDF_CLOB          *pArg5,
                     SQLUDF_CLOB          *pArg6,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_CLOB          *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pArg2Ind,
                     SQLUDF_NULLIND       *pArg3Ind,
                     SQLUDF_NULLIND       *pArg4Ind,
                     SQLUDF_NULLIND       *pArg5Ind,
                     SQLUDF_NULLIND       *pArg6Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_CHAR *pRetStr = NULL;
   int Retlen = 0; 

   //Declare elements enough for input arguments, here the input are 7
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Character Argument (pArg3),
   //Character Argument (pArg4),
   //Character Argument (pArg5),
   //Character Argument (pArg6),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[7];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data , 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pArg2->data , 
                                                  *pArg2Ind, 
                                                  pArg2->length, 
                                                  pDbInfo);

   *(pDataInArr + 2) = (CDB2Data*) new CDB2String(pArg3->data , 
                                                  *pArg3Ind, 
                                                  pArg3->length, 
                                                  pDbInfo);

   *(pDataInArr + 3) = (CDB2Data*) new CDB2String(pArg4->data , 
                                                  *pArg4Ind, 
                                                  pArg4->length, 
                                                  pDbInfo);

   *(pDataInArr + 4) = (CDB2Data*) new CDB2String(pArg5->data , 
                                                  *pArg5Ind, 
                                                  pArg5->length, 
                                                  pDbInfo);

   *(pDataInArr + 5) = (CDB2Data*) new CDB2String(pArg6->data , 
                                                  *pArg6Ind, 
                                                  pArg6->length, 
                                                  pDbInfo);

   //This is a string class because the parameter string is always a char
   *(pDataInArr + 6) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2String(pRetStr, 0, 0, pDbInfo);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         7,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         pRet->length = 0;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete *(pDataInArr + 3);
      delete *(pDataInArr + 4);
      delete *(pDataInArr + 5);
      delete *(pDataInArr + 6);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, &Retlen, NULL);
   
   pRet->length = (Retlen > (int)pRet->length)?pRet->length : Retlen;

   memcpy(pRet->data, pRetStr, pRet->length);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete *(pDataInArr + 3);
   delete *(pDataInArr + 4);
   delete *(pDataInArr + 5);
   delete *(pDataInArr + 6);
   delete pDataOutArr;
}

//---------------------------------------------------------------------------
// Function:   OptimMaskStrInt
//
//    Desc:    Accepts CLOB data argument and a parameter string
//             returns a integer
//
//-------------------------------------------------------------------------
//   Registration command for this function
//-------------------------------------------------------------------------
//Create  or replace function OptimMaskStr(
//Arg1 CLOB(800),
//ParamString CLOB(800) ) 
//returns INTEGER
//specific sOptimMaskStrInt
//external name '<UDF library path>!OptimMaskStrInt'
//Language C
//parameter style SQL
//not deterministic
//fenced threadsafe
//called on null input
//no sql
//external action
//scratchpad 10
//final call
//dbinfo
//allow parallel
//-------------------------------------------------------------------------
void OptimMaskStrInt(SQLUDF_CLOB          *pArg1,
                     SQLUDF_CLOB          *pParamString, 
                     SQLUDF_INTEGER       *pRet,
                     SQLUDF_NULLIND       *pArg1Ind,
                     SQLUDF_NULLIND       *pPSInd,
                     SQLUDF_NULLIND       *pRetInd,
                     SQLUDF_CHAR          *pSqlstate,
                     SQLUDF_CHAR          *pFname,
                     SQLUDF_CHAR          *pFspecname,
                     SQLUDF_CHAR          *pMsgtext,
                     SQLUDF_SCRATCHPAD    *pScratchpad,
                     SQLUDF_CALL_TYPE     *pCallType,
                     SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_INTEGER *pRetInt = NULL;

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[2];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data , 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   //This is a string class because the parameter string is always a char
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2Number(pRetInt, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         2,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetInteger(&pRetInt, pRetInd);
   
   *pRet = *pRetInt;
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;
}


//---------------------------------------------------------------------------
// Function:   OptimMaskStr2Int
//
//    Desc:    Accepts 2 CLOB data argument and a parameter string
//             returns a integer
//
//-------------------------------------------------------------------------
//   Registration command for this function
//-------------------------------------------------------------------------
//Create  or replace function OptimMaskStr2(
//Arg1 CLOB(800),
//Arg2 CLOB(800),
//ParamString CLOB(800) ) 
//returns INTEGER
//specific sOptimMaskStr2Int
//external name '<UDF library path>!OptimMaskStr2Int'
//Language C
//parameter style SQL
//not deterministic
//fenced threadsafe
//called on null input
//no sql
//external action
//scratchpad 10
//final call
//dbinfo
//allow parallel
//-------------------------------------------------------------------------
void OptimMaskStr2Int(  SQLUDF_CLOB          *pArg1,
                        SQLUDF_CLOB          *pArg2,
                        SQLUDF_CLOB          *pParamString, 
                        SQLUDF_INTEGER       *pRet,
                        SQLUDF_NULLIND       *pArg1Ind,
                        SQLUDF_NULLIND       *pArg2Ind,
                        SQLUDF_NULLIND       *pPSInd,
                        SQLUDF_NULLIND       *pRetInd,
                        SQLUDF_CHAR          *pSqlstate,
                        SQLUDF_CHAR          *pFname,
                        SQLUDF_CHAR          *pFspecname,
                        SQLUDF_CHAR          *pMsgtext,
                        SQLUDF_SCRATCHPAD    *pScratchpad,
                        SQLUDF_CALL_TYPE     *pCallType,
                        SQLUDF_DBINFO        *pDbInfo)
{
   RETVAL retVal = ODPPFAILURE;
   SQLUDF_INTEGER *pRetInt = NULL;

   //Declare elements enough for input arguments, here the input are 3
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Parameter String (pParamString)
   CDB2Data *pDataInArr[3];

   //Declare a pointer for output
   CDB2Data *pDataOutArr;

   //Allocate the input data classes
   //This is a string class because the input is a char
   *(pDataInArr + 0) = (CDB2Data*) new CDB2String(pArg1->data , 
                                                  *pArg1Ind, 
                                                  pArg1->length, 
                                                  pDbInfo);
   
   *(pDataInArr + 1) = (CDB2Data*) new CDB2String(pArg2->data , 
                                                  *pArg2Ind, 
                                                  pArg2->length, 
                                                  pDbInfo);

   //This is a string class because the parameter string is always a char
   *(pDataInArr + 2) = (CDB2Data*) new CDB2String(pParamString->data, 
                                                  *pPSInd, 
                                                  pParamString->length, 
                                                  pDbInfo);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (CDB2Data*) new CDB2Number(pRetInt, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pSqlstate,
                         pFname,
                         pMsgtext,
                         SQLUDF_MSGTEXT_LEN,
                         pScratchpad,
                         pCallType,
                         pDbInfo,
                         pDataInArr,
                         3,
                         pDataOutArr,
                         1);
   if(ODPPSUCCESS != retVal)
   {
      switch(retVal)
      {
      case ODPP_UDF_ERR_DB2_NULL_VALUE:
         *pRetInd = -1;
         break;
      }
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete pDataOutArr;
      return;      
   }
   //Get the output data
   pDataOutArr->GetInteger(&pRetInt, pRetInd);
   
   *pRet = *pRetInt;
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete pDataOutArr;
}



